## This plugin implements a printfilter
## selector for tkchooser
##
## Ethan Gold <etgold@cs.vassar.edu> 3/27/98

if {[debug]} { puts "loading Printfilter plugin.." }

## register the plugin with the main chooser
register_plugin misc printfilter PrintFilter

#### required functions to return plugin information
#### use these instead of globals
proc printfilter.geticon {} { return "printfilter.ppm" }
proc printfilter.getpubname {} { return "Printfilter" }
proc printfilter.getprotocol {} { return "misc" }

####### required functions that actually do stuff (maybe) #########

## start function
proc printfilter.start {} {
    ## read in config file and set options as global
    if {[debug]} {puts "printfilter: starting printfilter plugin"}
    printfilter.widgets "start"
    printfilter.readfilts
}

## stop function
proc printfilter.stop {} {
    ## global data area will automatically be cleared
    if {[debug]} {puts "printfilter: stopping printfilter plugin"}
    printfilter.widgets "stop"
}

## doubleclick callback
proc printfilter.doubleclick {} {
    ## get the value of the double-clicked item
    ## and pass it to the configurator
    printfilter.configfilt [get_curr_item]
}

## callback for new zone selection
proc printfilter.newzone {} {
    ## do nothing
}

####### end required functions ########

proc printfilter.readfilts {} {
    global userconfdir libdir
    set pluglist [plug_list]
    set plugframe [plug_frame]
    $plugframe.status configure -text "reading filters..."
    update

    printfilter.clearfilts

    set filters ""
    
    ## read global printfilters
    set fd [open "|/bin/ls $libdir/etc/" r]
    while {[gets $fd line] != -1} {
	foreach name $line {
	    if {[regexp {^printfilter\..+} $name]} {
		set name [lindex [split $name .] 1]
		set filters "$filters public:$name"
		if {[debug]} {puts "printfilter: found $name"}
	    }
	}
    }
    close $fd

    ## read personal printfilters
    set fd [open "|/bin/ls $userconfdir" r]
    while {[gets $fd line] != -1} {
	foreach name $line {
	    if {[regexp {^printfilter\..+} $name]} {
		set name [lindex [split $name .] 1]
		set filters "$filters private:$name"
		if {[debug]} {puts "printfilter: found $name"}
	    }
	}
    }
    close $fd

    set filters [lsort -dictionary $filters]
    foreach filter $filters {
	$pluglist insert end $filter
    }
    set_glob filters $filters

    $plugframe.status configure -text "ready."
    update
}

## procedure to build and destroy main
## plugin frame widgets
proc printfilter.widgets {command} {
    set plugframe [plug_frame]

    if {[string compare $command "start"] == 0} {
	
	button $plugframe.use -text "use" -command "printfilter.use"
	pack $plugframe.use

    } else {
	destroy $plugframe.use
    }

}

proc printfilter.use {} {
    global libdir userconfdir
    set filter [get_curr_item]

    ## set the filename for the filter
    set type [lindex [split $filter :] 0]
    set filtname [lindex [split $filter :] 1]

    if {[string compare $type "public"] == 0} {
	set filtfile "$libdir/etc/printfilter.$filtname"
    } else {
	set filtfile "$userconfdir/printfilter.$filtname"
    }
    
    catch {file delete $userconfdir/printfilter}
    file copy $filtfile $userconfdir/printfilter
    if {[string compare $filtfile $userconfdir/printfilter.$filtname] != 0} {
	catch {file delete $userconfdir/printfilter.$filtname}
	file copy $filtfile $userconfdir/printfilter.$filtname
    }

    printfilter.readfilts
}

## procedure to build a window to edit the
## name and commandline for the filter
proc printfilter.configfilt {filter} {
    global libdir userconfdir

    set cmdline ""
    set filtname ""

    ## read in the filter line from the file
    set type [lindex [split $filter :] 0]
    set filtname [lindex [split $filter :] 1]

    if {[string compare $type "public"] == 0} {
	set filtfile "$libdir/etc/printfilter.$filtname"
    } else {
	set filtfile "$userconfdir/printfilter.$filtname"
    }
    set fd [open $filtfile r]
    gets $fd cmdline
    if {[debug]} {puts "printfilter: cmdline for $filtname is \"$cmdline\""}
    close $fd
    
    set w .config
    catch {destroy $w}
    toplevel $w -class Dialog
    
    label $w.namelabel -text "Name:"
    entry $w.nameentry -textvariable filtname \
	    -width 40 -exportselection 0
    label $w.cmdlabel -text "Commandline - Must output to stdout"
    entry $w.cmdentry -textvariable cmdline \
	    -width 40 -exportselection 0
    button $w.cancel -text "Cancel" -command "destroy $w"
    button $w.ok -text "Ok" -command {} -default active

    pack $w.namelabel
    pack $w.nameentry
    pack $w.cmdlabel
    pack $w.cmdentry
    pack $w.cancel -side left -anchor s -padx 2 -pady 2
    pack $w.ok -side right -anchor s

    wm title $w "Configure $filter"
    update
}

proc printfilter.clearfilts {} {
    set pluglist [plug_list]
    $pluglist delete 0 end
}

#########
if {[debug]} {puts "Finished loading printfilter plugin."}







